
%  Copyright (c) 2013
%      Oliver van Kaick <ovankaic@gmail.com>

%
% Linearly map a set of values from one range to another
%
% new_val = map_func(val, new_mn, new_mx [, mn [, mx]])
%
% Input:
%   - val: vector or matrix containing the values to be mapped
%   - new_mn: minimum value of the new range
%   - new_mx: maximum value of the new range
%   - mn (optional): minimum value of the current range. If this
%   parameter is not provided, it is computed as min(val(:)). Setting a
%   minimum that is different from the actual data minimum allows to
%   clamp values that are too small.
%   - mx (optional): maximum value of the current range. If this
%   parameter is not provided, it is computed as max(val(:)). Setting a
%   maximum that is different from the actual data maximum allows to
%   clamp values that are too big.
%
% Output:
%   - new_val: vector or matrix (with the same dimensions and size as
%   the input 'val') storing the mapped values
%
% Description:
% This function linearly maps the values in the vector 'val' from the
% range [mn, mx] to the range [new_mn, new_mx], storing the result in
% the vector 'new_val'
%
function new_val = map_func(val, new_mn, new_mx, mn, mx)

% Check for missing parameters
if nargin < 5
    if issparse (val)
        mx = max(nonzeros(val));
    else
        mx = max(val(:));
    end
end

if nargin < 4
    if issparse(val)
        mn = min(nonzeros(val));
    else
        mn = min(val(:));
    end
end

% Clamp values according to given or computed min and max
if issparse(val)
    pos = find(val);
    val(pos(val(pos) < mn)) = mn;
    val(pos(val(pos) > mx)) = mx;
else
    val(val < mn) = mn;
    val(val > mx) = mx;
end

% Map values
if issparse(val)
    new_val = sparse(val);
    new_val(new_val ~= 0) = new_mn + (new_mx - new_mn)*((val(val ~= 0) - mn)./(mx - mn));
else
    new_val = new_mn + (new_mx - new_mn)*((val - mn)./(mx - mn));
end
