
%  Copyright (c) 2013
%      Noa Fish <noafish@post.tau.ac.il>

%%% calc patch adjacency as well as seam sets

%%% input:
% patches: cell array containing patches
% gr: KNN graph
% points: 3D point set
% normals: 3D normals corresponding to points
% outliers: outlier points
% force_conn: 0: do not force an isolated patch to have neighbors
%             1: force an isolated patch to have neighbors

%%% output:
% adj: patch adjacency matrix
% ang_hist: seam sets

function [adj, ang_hist] = patch_adjacency(patches, gr, points, normals, outliers, force_conn)

if ~exist('force_conn', 'var')
    force_conn = 0;
end

if(length(patches) == 1)
    adj = 0;
    ang_hist = 0;
    return;
end

clusts = patches;
labeling = clusters_to_clustering(patches);

nbins = 4;

adj = zeros(length(patches));
ang_hist = zeros(length(clusts), length(clusts), nbins);

for i=1:length(clusts)
    
    curr_clust = clusts{i};
    
    for j=1:length(curr_clust)
        
        currp = curr_clust(j);
        if(~isempty(find(outliers == currp)))
            continue;
        end
        point_neigh = gr{currp};
        point_neigh = setdiff(point_neigh, outliers);
        
        clu_neigh = labeling(point_neigh);
        
        other_clus = find(clu_neigh ~= i);
        other_neigh = point_neigh(other_clus);
        other_neigh_lbl = clu_neigh(other_clus);
        
        
        for k=1:length(other_neigh)
            
            otn = other_neigh(k);
            
            other_lbl = other_neigh_lbl(k);
            n1 = normals(currp,:);
            n2 = normals(otn,:);
            
            ang = calc_angle(points(currp, :), points(otn, :), n1, n2);
            ahist = ang_hist(i,other_lbl,:);
            
            % If angle is smaller than 0.05 then it's a light concavity
            if(ang <= 0.05)
                ahist(2) = ahist(2) + 1;
            end
            
            % If angle is between 0.05 and 0.5 it is a concavity
            if(ang > 0.05 && ang < 0.5)
                ahist(1) = ahist(1) + 1;
            end
            
            % If angle is 0.5 or larger it is a convexity
            if(ang >= 0.5)
                ahist(3) = ahist(3) + 1;
            end
            
            % total number of adjacencies
            ahist(4) = ahist(4) + 1;
            
            ang_hist(i,other_lbl,:) = ahist;
        end
        
        clu_neigh = unique(other_neigh_lbl);
        adj(i,clu_neigh) = 1;
    end
    
    if(force_conn == 1 && isempty(find(adj(i,:) == 1)))
        
        dists_to_clusts = [];
        for j=1:length(clusts)
            if(j == i)
                continue;
            end
            other_clust = clusts{j};
            [nn, dists] = knnsearch(points(other_clust,:),points(curr_clust,:),'K',1);
            [md, mind] = min(dists);
            dists_to_clusts = [dists_to_clusts; j md];
        end
        
        srtd = sortrows(dists_to_clusts, 2);
        adj(i,srtd(1,1)) = 1;
        adj(srtd(1,1),i) = 1;
        adj(i,srtd(2,1)) = 1;
        adj(srtd(2,1),i) = 1;
        ang_hist(i,srtd(1,1),4) = -1;
        ang_hist(srtd(1,1),i,4) = -1;
        ang_hist(i,srtd(2,1),4) = -1;
        ang_hist(srtd(2,1),i,4) = -1;
    end
end


end


