
%  Copyright (c) 2013
%      Noa Fish <noafish@post.tau.ac.il>

%%% compute visibility between neighboring patches

%%% input:
% points: 3D point set
% normals: 3D normals corresponding to points
% patches: cell array containing the candidate points of all patches
% patch_adj_c: cell array containing patch adjacency
% gr: KNN graph
% outliers: outlier points

%%% output:
% los_matrix: matrix containing the number of cast rays that hit the
%             correct target
% ilos_matrix: matrix containing the number of cast rays that didn't hit
%              the correct target


function [los_matrix, ilos_matrix] = patch_los_matrix_neigh(points, normals, patches, patch_adj, gr, outliers)


npatches = length(patches);
N = size(points,1);

los_matrix = zeros(npatches);
ilos_matrix = los_matrix;

splatRadiuses = point_cloud_estimate_radiuses(points, normals);
splatRadiusesSquare = splatRadiuses.^2; % useful later on


for i=1:npatches
    
    source_points = patches{i};
    
    for k=1:length(source_points)
    
        src = source_points(k);
        srcp = points(src,:);
        srcn = normals(src,:);
        
        repNormal = repmat(srcn, [N,1]);
        repSource = repmat(srcp, [N,1]);
        faceFronters = sum((points - repSource) .* repNormal, 2) < 0; % all the points that are in front of this point
        
        neigh = patch_adj{i};
        
        for ne=1:length(neigh)
            
            j = neigh(ne);
            
            if(j < i)
                continue;
            end
            
            target_points = patches{j};
            
            
            for l=1:length(target_points)
                
                tgt = target_points(l);
                
                % cast ray between src and tgt and receive the hit point
                hit_point = cast_ray(points, normals, splatRadiusesSquare, faceFronters, src, tgt, gr, outliers);
                
                % if no point was hit, check the geodesic path between src
                % and tgt - if the ray between src and tgt does not go out
                % of this path then consider this ray as a good hit
                if(hit_point == -1)
                    [dists, path, succ] = st_shortest_path(points, gr, src, tgt);
                    if(succ == 1)
                        valid = check_geodesic_path(points, normals, src, tgt, path);
                        if(valid)
                            hit_point = tgt;
                        end
                    end
                end
                
                % if the hit_point was not tgt, check whether the hit point
                % and tgt share mutual neighbors. if so, check whether the
                % ray between the hit point and tgt goes outside the
                % geodesic path between them. if not - consider this ray as
                % a good hit
                if(hit_point ~= tgt && hit_point ~= -1)
                    ints = intersect(gr{tgt}, gr{hit_point});
                    if(~isempty(ints))
                        [dists, path, succ] = st_shortest_path(points, gr, hit_point, tgt);
                        if(succ == 1)
                            valid = check_geodesic_path(points, normals, hit_point, tgt, path);
                            if(valid)
                                hit_point = tgt;
                            end
                        end
                    end
                end
                
                
                if(hit_point == tgt)
                    los_matrix(i,j) = los_matrix(i,j) + 1;
                    los_matrix(j,i) = los_matrix(j,i) + 1;
                else
                    ilos_matrix(i,j) = ilos_matrix(i,j) + 1;
                    ilos_matrix(j,i) = ilos_matrix(j,i) + 1;
                end
                    
                
            end
        end
    end
end


end



